@@ -105,7 +105,7 @@ Bool
 xf86VGAarbiterAllowDRI(ScreenPtr pScreen)
 {
     int vga_count;
-    int rsrc_decodes;
+    int rsrc_decodes = 0;
     ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
 
     if (vga_no_arb)
@@ -257,7 +257,7 @@ VGAarbiterCloseScreen(ScreenPtr pScreen)
     UNWRAP_SCREEN_INFO(FreeScreen);
     UNWRAP_SPRITE;
 
-    free((pointer) pScreenPriv);
+    free((void *) pScreenPriv);
     xf86VGAarbiterLock(xf86ScreenToScrn(pScreen));
     val = (*pScreen->CloseScreen) (pScreen);
     xf86VGAarbiterUnlock(xf86ScreenToScrn(pScreen));
@@ -265,23 +265,21 @@ VGAarbiterCloseScreen(ScreenPtr pScreen)
 }
 
 static void
-VGAarbiterBlockHandler(ScreenPtr pScreen,
-                       pointer pTimeout, pointer pReadmask)
+VGAarbiterBlockHandler(ScreenPtr pScreen, void *pTimeout)
 {
     SCREEN_PROLOG(BlockHandler);
     VGAGet(pScreen);
-    pScreen->BlockHandler(pScreen, pTimeout, pReadmask);
+    pScreen->BlockHandler(pScreen, pTimeout);
     VGAPut();
     SCREEN_EPILOG(BlockHandler, VGAarbiterBlockHandler);
 }
 
 static void
-VGAarbiterWakeupHandler(ScreenPtr pScreen, unsigned long result,
-                        pointer pReadmask)
+VGAarbiterWakeupHandler(ScreenPtr pScreen, int result)
 {
     SCREEN_PROLOG(WakeupHandler);
     VGAGet(pScreen);
-    pScreen->WakeupHandler(pScreen, result, pReadmask);
+    pScreen->WakeupHandler(pScreen, result);
     VGAPut();
     SCREEN_EPILOG(WakeupHandler, VGAarbiterWakeupHandler);
 }
@@ -323,9 +321,8 @@ VGAarbiterSourceValidate(DrawablePtr pDrawable,
 
     SCREEN_PROLOG(SourceValidate);
     VGAGet(pScreen);
-    if (pScreen->SourceValidate)
-        (*pScreen->SourceValidate) (pDrawable, x, y, width, height,
-                                    subWindowMode);
+    (*pScreen->SourceValidate) (pDrawable, x, y, width, height,
+                                subWindowMode);
     VGAPut();
     SCREEN_EPILOG(SourceValidate, VGAarbiterSourceValidate);
 }
@@ -586,7 +583,7 @@ VGAarbiterCopyGC(GCPtr pGCSrc, unsigned long mask, GCPtr pGCDst)
 }
 
 static void
-VGAarbiterChangeClip(GCPtr pGC, int type, pointer pvalue, int nrects)
+VGAarbiterChangeClip(GCPtr pGC, int type, void *pvalue, int nrects)
 {
     GC_UNWRAP(pGC);
     (*pGC->funcs->ChangeClip) (pGC, type, pvalue, nrects);
@@ -860,7 +857,7 @@ VGAarbiterImageGlyphBlt(DrawablePtr pDraw,
                         GCPtr pGC,
                         int xInit, int yInit,
                         unsigned int nglyph,
-                        CharInfoPtr * ppci, pointer pglyphBase)
+                        CharInfoPtr * ppci, void *pglyphBase)
 {
     ScreenPtr pScreen = pGC->pScreen;
 
@@ -877,7 +874,7 @@ VGAarbiterPolyGlyphBlt(DrawablePtr pDraw,
                        GCPtr pGC,
                        int xInit, int yInit,
                        unsigned int nglyph,
-                       CharInfoPtr * ppci, pointer pglyphBase)
+                       CharInfoPtr * ppci, void *pglyphBase)
 {
     ScreenPtr pScreen = pGC->pScreen;
 
