@@ -1,4 +1,5 @@
 /*
+ * Copyright (c) 2002-2012 Apple Inc. All rights reserved.
  * Copyright (c) 2003-2004 Torrey T. Lyons. All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -27,32 +28,38 @@
 #ifndef QUARTZ_KEYBOARD_H
 #define QUARTZ_KEYBOARD_H 1
 
-#define XK_TECHNICAL		// needed to get XK_Escape
+#define XK_TECHNICAL      // needed to get XK_Escape
 #define XK_PUBLISHING
 #include "X11/keysym.h"
 #include "inputstr.h"
 
-#include <pthread.h>
-
 // Each key can generate 4 glyphs. They are, in order:
 // unshifted, shifted, modeswitch unshifted, modeswitch shifted
-#define GLYPHS_PER_KEY  4
-#define NUM_KEYCODES    248	// NX_NUMKEYCODES might be better
-#define MIN_KEYCODE     XkbMinLegalKeyCode     // unfortunately, this isn't 0...
-#define MAX_KEYCODE     NUM_KEYCODES + MIN_KEYCODE - 1
+#define GLYPHS_PER_KEY 4
+#define NUM_KEYCODES   248      // NX_NUMKEYCODES might be better
+#define MIN_KEYCODE    XkbMinLegalKeyCode      // unfortunately, this isn't 0...
+#define MAX_KEYCODE    NUM_KEYCODES + MIN_KEYCODE - 1
 
 /* These functions need to be implemented by Xquartz, XDarwin, etc. */
-Bool QuartsResyncKeymap(Bool sendDDXEvent);
+Bool
+QuartsResyncKeymap(Bool sendDDXEvent);
 
 /* Provided for darwinEvents.c */
-void DarwinKeyboardReloadHandler(void);
-int DarwinModifierNXKeycodeToNXKey(unsigned char keycode, int *outSide);
-int DarwinModifierNXKeyToNXKeycode(int key, int side);
-int DarwinModifierNXKeyToNXMask(int key);
-int DarwinModifierNXMaskToNXKey(int mask);
-int DarwinModifierStringToNXMask(const char *string, int separatelr);
+void
+DarwinKeyboardReloadHandler(void);
+int
+DarwinModifierNXKeycodeToNXKey(unsigned char keycode, int *outSide);
+int
+DarwinModifierNXKeyToNXKeycode(int key, int side);
+int
+DarwinModifierNXKeyToNXMask(int key);
+int
+DarwinModifierNXMaskToNXKey(int mask);
+int
+DarwinModifierStringToNXMask(const char *string, int separatelr);
 
 /* Provided for darwin.c */
-void DarwinKeyboardInit(DeviceIntPtr pDev);
+void
+DarwinKeyboardInit(DeviceIntPtr pDev);
 
 #endif /* QUARTZ_KEYBOARD_H */
