@@ -67,7 +67,7 @@ winAllocatePrivates(ScreenPtr pScreen)
     /* Initialize the memory of the private structure */
     ZeroMemory(pScreenPriv, sizeof(winPrivScreenRec));
 
-    /* Intialize private structure members */
+    /* Initialize private structure members */
     pScreenPriv->fActive = TRUE;
 
     /* Register our screen private */
@@ -79,13 +79,6 @@ winAllocatePrivates(ScreenPtr pScreen)
     /* Save the screen private pointer */
     winSetScreenPriv(pScreen, pScreenPriv);
 
-    /* Reserve GC memory for our privates */
-    if (!dixRegisterPrivateKey
-        (g_iGCPrivateKey, PRIVATE_GC, sizeof(winPrivGCRec))) {
-        ErrorF("winAllocatePrivates - AllocateGCPrivate () failed\n");
-        return FALSE;
-    }
-
     /* Reserve Pixmap memory for our privates */
     if (!dixRegisterPrivateKey
         (g_iPixmapPrivateKey, PRIVATE_PIXMAP, sizeof(winPrivPixmapRec))) {
